/* 
   LzmaTest.c
   Test application for LZMA Decoder
   LZMA SDK 4.01 Copyright (c) 1999-2004 Igor Pavlov (2004-02-15)
 */

#include <stdio.h>
#include <string.h>
#include "LzmaDecode.h"
#include "LzmaKWrapper.h"

/*#define LZMAKWRAPPER_TRACE*/
#ifdef LZMAKWRAPPER_TRACE
#define TRACE(s, args...)		do { \
						printf("mtdx: "s, ## args); \
					} while(0)
#else
#define TRACE(s, args...)
#endif

#define ERROR(s, args...)		do { \
						fprintf(stderr, s, ## args); \
					} while(0)

#define DEF_DICTIONARY_SIZE               0x200000
#define DEF_LITERAL_POS_STATE_BITS        0
#define DEF_LITERAL_CONTEXT_BITS          3
#define DEF_POS_STATE_BITS                2

static int	internal_size;
static unsigned char	*internal_data=0;
//static int	dictionary_size;

int lzma_workspace_size(void)
{
//	int pb=2;
	int lp=0;
	int lc=3;

	return (LZMA_BASE_SIZE + (LZMA_LIT_SIZE << (lc + lp)))* sizeof(CProb);
}

int lzma_init(unsigned char *data, int size)
{
	internal_data=data;
	internal_size=size;
	return 0;
}

int lzma_inflate(char *source, int s_len, char *dest, unsigned int *d_len)
{
	/* Get the properties */
	unsigned int check_internal_size;
	unsigned char properties[5];
	unsigned char prop0;
	int lc, lp, pb;
//	int	dest_size;
	int res, i;
	unsigned int dictionary_size;
	unsigned int encoded_size;

	if(s_len<5){
		/* Can't even get properities, just exit */
		return LZMA_ERROR;
	}

	/* Get the size of the uncompressed buffer */
	memcpy(&encoded_size, source + 0x14, 4);

	/* The LZMA-compressed kernel doesn't seem to include this header data,
	 * or at least if it does I have no fucking idea where to find it. So we
	 * replace this section of the code with simply assigning default values.
	 * Which is shit, but it seems to work. */
#ifdef notdef
	/* We use this to check that the size of internal data structures
	   are big enough. If it isn't, then we flag it. */
	memcpy(properties, source, sizeof(properties));
	prop0 = properties[0];
	if (prop0 >= (9*5*5)) {
		fprintf(stderr, "prop0 = %02.2x\n", prop0);
		return LZMA_ERROR;
	}
	for (pb = 0; prop0 >= (9 * 5); 
			pb++, prop0 -= (9 * 5));
	for (lp = 0; prop0 >= 9; 
			lp++, prop0 -= 9);
	lc = prop0;
	source += 5;
#else
	dictionary_size = DEF_DICTIONARY_SIZE;
	lp = DEF_LITERAL_POS_STATE_BITS;
	lc = DEF_LITERAL_CONTEXT_BITS;
	pb = DEF_POS_STATE_BITS;
#endif
	
	
	TRACE("Uncompressed size: %x, input size: %x\n", encoded_size, s_len);
	
	check_internal_size = 
		(LZMA_BASE_SIZE + (LZMA_LIT_SIZE << (lc + lp)))* sizeof(CProb);

#ifdef notdef
	for (i = 0; i < 4; i++)
		dictionary_size += (UInt32)(properties[1 + i]) << (i * 8);
#endif
	
	TRACE("internal_size = %d, check_internal_size = %d\n", internal_size, check_internal_size);
	TRACE("lc = %d, lp=%d, pb=%d\n", lc, lp, pb);
	TRACE("byte=%x, dictionary size = %8.8x\n", prop0, dictionary_size);
	if(check_internal_size > internal_size){
		return LZMA_TOO_BIG;
	}

//	dest_size=*d_len;
	TRACE("inflating encoded size = %x\n", encoded_size);
	res = LzmaDecode(internal_data, internal_size,
			lc, lp, pb,
			(unsigned char *)source + 0x20, s_len,
			(unsigned char *)dest, encoded_size, d_len);

	return res;
}
